'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = applyRules;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _lodashForeach = require('lodash.foreach');

var _lodashForeach2 = _interopRequireDefault(_lodashForeach);

/**
 * @param {Array<RawDiff>} diffs Result of diff module
 * @param {Map} breakRules  map of breaking rules indexed by their id
 * @param {Map} smoothRules map of smooth rules indexed by their id
 * @return {
 *   breaks: Array<Diff>
 *   smooths: Array<Diff>,
 *   unmatchDiffs: Array<RawDiff>
 * }
 * @note Diff: {ruleId: String, message: String}
 */

function applyRules(diffs) {
  var breakRules = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var smoothRules = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  var breaks = [];
  var smooths = [];
  var unmatchDiffs = [];

  (0, _lodashForeach2['default'])(diffs, function (diff) {
    var matchRule = false;

    (0, _lodashForeach2['default'])(breakRules, function (rule, ruleId) {
      var ruleResult = rule(diff);
      if (ruleResult) {
        matchRule = true;
        breaks = breaks.concat({
          ruleId: ruleId,
          message: ruleResult
        });
        return false; // break
      }
    });

    if (matchRule) {
      return true; // continue
    }

    (0, _lodashForeach2['default'])(smoothRules, function (rule, ruleId) {
      var ruleResult = rule(diff);
      if (ruleResult) {
        matchRule = true;
        smooths = smooths.concat({
          ruleId: ruleId,
          message: ruleResult
        });
        return false; // break
      }
    });

    if (matchRule) {
      return true; // continue
    }

    unmatchDiffs = unmatchDiffs.concat(diff);
  });

  return {
    breaks: breaks,
    smooths: smooths,
    unmatchDiffs: unmatchDiffs
  };
}

module.exports = exports['default'];