'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = prepareSpec;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _dereference = require('./dereference');

var _dereference2 = _interopRequireDefault(_dereference);

var _indexParameters = require('./indexParameters');

var _indexParameters2 = _interopRequireDefault(_indexParameters);

var _inlineGlobals = require('./inlineGlobals');

var _inlineGlobals2 = _interopRequireDefault(_inlineGlobals);

var _inlineParameters = require('./inlineParameters');

var _inlineParameters2 = _interopRequireDefault(_inlineParameters);

var _inlineRequiredProperties = require('./inlineRequiredProperties');

var _inlineRequiredProperties2 = _interopRequireDefault(_inlineRequiredProperties);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

/**
 * @param  {string|object} spec
 * @return {Promise}
 */

function prepareSpec(spec) {
  var debug = require('debug')('swagger-diff:workflow:prepareSpec');

  debug('start');
  if (typeof spec === 'string' && spec.indexOf('http') !== 0 && !_path2['default'].isAbsolute(spec)) {
    spec = _path2['default'].resolve(process.cwd(), spec); // eslint-disable-line no-param-reassign
  }

  return (0, _dereference2['default'])(spec).then(function (dereferencedSpec) {
    debug('dereferenced');
    var specs = dereferencedSpec;

    specs = (0, _inlineGlobals2['default'])(specs);
    debug('globals inlined');

    specs = (0, _inlineParameters2['default'])(specs);
    debug('parameters inlined');

    specs = (0, _indexParameters2['default'])(specs);
    debug('parameters indexed');

    specs = (0, _inlineRequiredProperties2['default'])(specs);
    debug('required properties inlined');

    return specs;
  });
}

module.exports = exports['default'];